| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
Text.PariPari
Synopsis
- module Text.PariPari.Class
- module Text.PariPari.Combinators
- module Text.PariPari.Acceptor
- module Text.PariPari.Reporter
- runParser :: Parser a -> FilePath -> ByteString -> Either Report a
- runSeqParser :: Parser a -> FilePath -> ByteString -> Either Report a
- runParserWithOptions :: ReportOptions -> Parser a -> FilePath -> ByteString -> Either Report a
- runSeqParserWithOptions :: ReportOptions -> Parser a -> FilePath -> ByteString -> Either Report a
Documentation
module Text.PariPari.Class
module Text.PariPari.Combinators
module Text.PariPari.Acceptor
module Text.PariPari.Reporter
runParser :: Parser a -> FilePath -> ByteString -> Either Report a Source #
Run fast Acceptor and slower Reporter on the given ByteString **in parallel**.
The FilePath is used for error reporting.
When the acceptor does not return successfully, the result from the reporter
is awaited.
runSeqParser :: Parser a -> FilePath -> ByteString -> Either Report a Source #
Run fast Acceptor and slower Reporter on the given ByteString **sequentially**.
The FilePath is used for error reporting.
When the acceptor does not return successfully, the result from the reporter
is awaited.
runParserWithOptions :: ReportOptions -> Parser a -> FilePath -> ByteString -> Either Report a Source #
Run parsers **in parallel** with additional ReportOptions.
runSeqParserWithOptions :: ReportOptions -> Parser a -> FilePath -> ByteString -> Either Report a Source #
Run parsers **sequentially** with additional ReportOptions.